#include "entity.h"

#include <cmath>
#include "SDL_opengl.h"

#include "random.h"
#include "glutsolidsphere.h"
#include "model.h"

namespace el2 {


Entity::Entity() {
	init();
}

void Entity::init() {
	x=0.0f;
	y=0.0f;
	z=0.0f;
	remove=false;
	through=false;
	movement=M_NONE;
	type=T_PLANE;
	movdat=0;
	rotx=0.0f;
	roty=0.0f;
	rotz=0.0f;
	num=0;
	exploding=0;
	surrounding[S_PX]=NULL;
	surrounding[S_PZ]=NULL;
	surrounding[S_NX]=NULL;
	surrounding[S_NZ]=NULL;
}

void Entity::draw() {
	if (remove) return;

	glPushMatrix();
	glTranslatef(x,y,z);

	if (exploding>0) {
		glColor3f(1.0f,0.0f,0.0f);
		glScalef(0.1f*exploding/2,0.1f*exploding/2,0.1f*exploding/2);
		glutSolidSphere(1.0,10,10);
		glPopMatrix();
		return;
	}
	glRotatef(rotx,-90.0f,0.0f,0.0f);
	glRotatef(roty,0.0f,-90.0f,0.0f);
	glRotatef(rotz,0.0f,0.0f,-90.0f);

	bool px=true,nx=true,pz=true,nz=true;
	if (type==T_CARRIER_PART&&isactive()){
		px=false;
		if (surrounding[S_PX]==NULL||!surrounding[S_PX]->isactive()) { px=true;}
		nx=false;
		if (surrounding[S_NX]==NULL||!surrounding[S_NX]->isactive()) { nx=true;}
		pz=false;
		if (surrounding[S_PZ]==NULL||!surrounding[S_PZ]->isactive()) { pz=true;}
		nz=false;
		if (surrounding[S_NZ]==NULL||!surrounding[S_NZ]->isactive()) { nz=true;}
	}
	switch (type) {
		case T_CARRIER_PART:
			Model::carrier_part(px,nx,pz,nz);
			break;
		case T_CARRIER_SIDE1:
			Model::carrier_side1(px,nx,pz,nz);
			break;
		case T_CARRIER_SIDE2:
			Model::carrier_side2(px,nx,pz,nz);
			break;
		case T_CARRIER_SIDE3:
			Model::carrier_side3(px,nx,pz,nz);
			break;
		case T_CARRIER_SIDE4:
			Model::carrier_side4(px,nx,pz,nz);
			break;
		case T_CARRIER_CORE:
			glColor3f(0.0f,0.0f,0.0f);
			glutSolidSphere(1.0,10,10);
			break;
		default:
			Model::plane();
			break;
	}





	glPopMatrix();
}
void Entity::tick(int ticks) {
	if (exploding>0) {
		exploding++;
		if (exploding>60) {
			remove=true;
			return;
		}
		return;
	}
	switch (movement) {
		case M_LINE:
			break;
		case M_ROLL:
			rotz+=0.01f;
			break;
		case M_SWING:
			static const float swing=30.0f;
			if (movdat) {
				roty-=0.01f;
				if (roty<-swing) {
					movdat=!movdat;
				} 
			} else {
				roty+=0.01f;
				if (roty>swing) {
					movdat=!movdat;
				} 
			}
			break;
		case M_LOWER:
			if (y>4.0f) {
				rotx=-5.0f;
			} else {
				rotx=0.0f;
			}
			break;
		case M_SPLIT:
			if (z<-40.0f) break;
			if (num>0) {
				roty=-25.0f;
			} 
			if (num<0) {
				roty=+25.0f;
			}
			break;
		case M_TUMBLE: 
			if (rotx>-70.0f) rotx-=0.01f; 
			if (movdat>0) { movdat--; break; }
			if (rotx>-70.0f) rotx-=0.8f; 
			roty+=0.11f; 
			y-=0.01f;
			break;
		case M_TUMBLE2: 
			if (rotx>-70.0f) rotx-=0.01f; 
			if (movdat>0) { movdat--; break; }
			if (rotx>-70.0f) rotx-=0.9f; 
			roty-=0.11f; 
			y-=0.01f;
			break;
		default:
			break;
	};

	static const float degtorad = M_PI/180;

	float dirx,diry,dirz;

	dirx=-(float)sinf(roty*degtorad);
	diry= (float)sinf(rotx*degtorad);
	dirz= (float)cosf(roty*degtorad);


	x+=dirx*0.01;
	y+=diry*0.01;
	z+=dirz*0.01;

	if (z>20) {
		through=true;
		remove=true;
	}
	if (y<=-4.5f) {
		exploding=1;
	}
}


bool Entity::isactive() {
	if (remove) return false;
	if (exploding>0) return false;
	if (movement==M_TUMBLE) return false;
	if (movement==M_TUMBLE2) return false;
	return true;
}


void Entity::hit(bool doexplode) {
	if (movement==M_TUMBLE) {
		exploding=1;
		return;
	}
	if (movement==M_TUMBLE2) {
		exploding=1;
		return;
	}
	if (!isactive()) return;
	if (type==T_CARRIER_CORE) {
		//std::cout << "hit carrier core " << waveid << std::endl;
		exploding=1;
		hit_all(waveid);
		exploding=0;
	}


	if (doexplode||Random::instance()->get(15)==0) {
		exploding=1;
		return;
	}

	if (Random::instance()->get(2)==0) {
		movement=M_TUMBLE;
	} else {
		movement=M_TUMBLE2;
	}

	if (Random::instance()->get(6)==0) {
		movdat=Random::instance()->get(5000);
	}
	if (Random::instance()->get(2)==0) {
		rotz-=(float)Random::instance()->get(140)/10.0f;
	} else {
		rotz+=(float)Random::instance()->get(140)/10.0f;
	}
	rotx=-(float)Random::instance()->get(80)/10.0f;
	if (Random::instance()->get(2)==0) {
		roty=-(float)Random::instance()->get(80)/10.0f;
	}
	
}
void Entity::clone(Entity *e) {
	x=e->x;
	y=e->y;
	z=e->z;
	movement=e->movement;
	waveid=e->waveid;
}

std::vector < Entity > entities;



void Entity::hit_all(int id) {
	std::vector <Entity>::iterator it;
	
	for (it=entities.begin();it!=entities.end();it++) {
		if ((*it).waveid!=id) continue;
		(*it).hit();
	}
}

void Entity::draw_all() {
	std::vector <Entity>::iterator it;
	
	for (it=entities.begin();it!=entities.end();it++) {
		(*it).draw();
	}
}
Entity *Entity::select(int x,int y) {

   glDrawBuffer(GL_BACK);
   glReadBuffer(GL_BACK);

	Entity *selected=NULL;
	selected=select_single(x,y);
	if (selected!=NULL) return selected;

	static const int size=2;
	static const int vals[]={0,3,-3,-6,6};
	int xv,yv;

	for (int ix=-size;ix<=size;ix++) {
		for (int iy=-size;iy<=size;iy++) {
			xv=vals[ix+size];	
			yv=vals[iy+size];	
			if (xv==0&&yv==0) continue;

			selected=select_single(x+xv,y+yv);
			if (selected!=NULL) return selected;
		}
	}

	return NULL;
}
Entity *Entity::select_single(int x,int y) {
	std::vector <Entity>::iterator it;

   glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);


	Entity *selected=NULL;
	unsigned int p;
	unsigned int rem=std::numeric_limits<unsigned int>::max();

	for (it=entities.begin();it!=entities.end();it++) {
		if ((*it).remove) continue;
		(*it).draw();
		glReadPixels(x,y,1,1,GL_DEPTH_COMPONENT, GL_UNSIGNED_INT,(GLvoid *)&p);
		if (p>=rem) continue;
		rem=p;
		selected=&(*it);
	}
	return selected;
}


void Entity::reset() {
	entities.resize(0);
}

int Entity::clear() {
	std::vector <Entity>::iterator it;

	int ret=0;
	for (it=entities.begin();it!=entities.end();it++) {
		if (!(*it).remove) return -1;
		if ((*it).through) ret++;
	}
	entities.resize(0);
	return ret;
}

void Entity::tick_all(int ticks) {
	std::vector <Entity>::iterator it;

	for (it=entities.begin();it!=entities.end();it++) {
		for (int i=0;i<ticks;i++) {
			(*it).tick(ticks);
		}
	}
}
Entity *Entity::add(Entity e) {
	entities.push_back(e);
	return &(entities[entities.size()-1]);
}



std::ostream& operator<<(std::ostream&s, Entity sk) {
	return s << sk.x << "x" << sk.y << "x" << sk.z;
}


} //namespace
