#include "level.h"

#include <sstream>
#include "SDL.h"
#include "SDL_opengl.h"

#include "font_all.h"
#include "random.h"
#include "entity.h"
#include "formation.h"

namespace el2 {


static unsigned int level;
static unsigned int nextwave;
static unsigned int wavecount;
static unsigned int through;
static bool wait;
static bool started_;


void Level::reset() {
	level=0;
	wait=true;
	nextwave=0;
	wavecount=0;
	through=0;
	started_=false;
	Entity::reset();
}
bool Level::started() {
	return started_;
}

void Level::start() {
	started_=true;
}

static std::vector < std::vector < E_Movement > > movements;
static void register_movement(E_Movement f,unsigned int start,unsigned int end) {
	if (end==0||end>movements.size()) {
		end=movements.size();
	}
	for (unsigned int i=start;i<end;i++) { 
		movements[i].push_back(f); 
	}
}

static E_Movement get_movement(unsigned int level) {
	if (level>=movements.size()) {
		level=1+Random::instance()->get(movements.size());
	}
	if (level>3&&Random::instance()->get(7)==0) {
		level=1+Random::instance()->get(level-1);
	}

	if (movements[level-1].size()==0) {
		std::cout << "no movements for " << level << std::endl;
		return M_LINE;
	}

	int r=Random::instance()->get(movements[level-1].size());

	E_Movement ret=movements[level][r];
	return ret;
}


static std::vector < std::vector < E_Formation > > formations;
void register_formation(E_Formation f,unsigned int start,unsigned int end) {
	if (end==0||end>formations.size()) {
		end=formations.size();
	}
	for (unsigned int i=start;i<end;i++) { formations[i].push_back(f); }
}

E_Formation get_formation(unsigned int level) {
	if (level>=formations.size()) {
		level=1+Random::instance()->get(formations.size()-1);
	} else if (level>3&&Random::instance()->get(7)==0) {
		level=1+Random::instance()->get(level-1);
	}

	if (formations[level-1].size()==0) {
		std::cout << "no formations for " << level << std::endl;
		return F_LINE;
	}

	int r=Random::instance()->get(formations[level-1].size());

	E_Formation ret=formations[level][r];
	return ret;
}

void Level::init() {
	formations.resize(11);
	register_formation(F_V3,0,3);
	register_formation(F_LINE,2,4);
	register_formation(F_V,3,5);
	register_formation(F_IV,4,6);
	register_formation(F_N,5,7);
	register_formation(F_DOUBLE_V,6,8);
	register_formation(F_3V,8,9);
	register_formation(F_CARRIER,9,10);


	movements.resize(11);
	register_movement(M_LINE,0,4);
	register_movement(M_ROLL,1,8);
	register_movement(M_LOWER,3,8);
	register_movement(M_SWING,5,10);
	register_movement(M_SPLIT,6,10);

	reset();
}


void startpos(Entity *e) {
	e->y=4.0f+(float)(Random::instance()->get(120))/10;;
	e->z=-150.0f;
	e->x=-20.0f+(float)(Random::instance()->get(400))/10;
}



unsigned int defense() {
	return 0+level*5;
}


void Level::tick() {
	if (wait) {
		int t=Entity::clear();
		if (t==-1) { return; }
		through+=t;
		level++;
		wavecount=0;
		wait=false;
	}

	if (SDL_GetTicks()<nextwave) return;
	if (wavecount==7) { wait=true; return; }
	wavecount++;

	int delay=3000-level*50;
	if (delay<1000) delay=1000;
	nextwave=SDL_GetTicks()+delay;

	Entity e;
	e=Entity(get_movement(level),wavecount);
	startpos(&e);
	E_Formation form;

	form=get_formation(level);
	formation(form,&e);
	Entity::add(e);
}




void write(SDL_Rect pos,const char *text) {
	int index;
	SDL_Rect p;
	for (unsigned int i=0;i<strlen(text);i++) {
		char c=text[i];
		p.x=pos.x+i*FONT_ALL_DX;
		p.y=pos.y;
		p.w=1;
		p.h=1;

		index=c-33;
		if (index<0||index>94) { continue; }
		for (int l=0;l<14;l++) {
			p.y=pos.y+l;
			const char *str=font_all[index][l];
			glColor3f(1.0f,1.0f,1.0f);
			for (unsigned int c=0;c<strlen(str);c++) {
				if (str[c]==' ') continue;
				p.x=pos.x+i*FONT_ALL_DX+c;
				glPushMatrix();
				glTranslatef(p.x/50.0f-1.0f,-p.y/50.0f,0+1.0f);
				glScalef(0.02f,0.02f,0.02f);
				glBegin(GL_QUADS);
		
				glVertex2f(0.0f,0.0f);
				glVertex2f(1.0f,0.0f);
				glVertex2f(1.0f,1.0f);
				glVertex2f(0.0f,1.0f);
				glEnd();
				glPopMatrix();
			}
		}
	}
}

bool Level::gameover() {
	if (through>defense()) {
		return true;
	}
	return false;
}

static unsigned int lastleveldrawn;
static GLuint dl=0;

void Level::draw() {
	SDL_Rect r; r.x=0; r.y=0; r.w=0; r.h=0;

	if (lastleveldrawn!=level&&dl!=0) {
		glDeleteLists(dl,1);
		dl=0;
	}

   if (dl!=0) { 
		glCallList(dl); 
		return; 
	}
   dl = glGenLists(1);
   glNewList(dl,GL_COMPILE_AND_EXECUTE);

	lastleveldrawn=level;



	std::ostringstream sstr;

	sstr << "Level: " << level;

	if (!started()) {
		sstr	 << " Click to start. Click to shoot enemies.";
	} else if (gameover()) {
		sstr	 << "  Game over. Click to return.";
	} else {
		sstr << " through: " << through;
		sstr	 << "  Defense left: " << (defense()-through);
	}

	glPushMatrix();
	glTranslatef(-2.0f,-2.0f,-5.0f);

	write(r,sstr.str().c_str());
	glPopMatrix();

	glEndList();
}

std::ostream& operator<<(std::ostream&s, Level sk) {
	return s;
}

} //namespace
